<?php
namespace Compras\Logic\Fornecimento;

use \Geral\Util\NovoCodigo;
use \Geral\Entities\Itemautf;
use \Geral\Repositories\Marca;
use \Geral\Entities\Comprador;
use \Geral\Entities\Fornecedor;
use \Geral\Entities\Prazopgtforn;
use \Geral\Repositories\Produto;
use \Planejamento\Entities\Autoforne;
use Setforms2\Runtime\Framework;
use \Suprimentos\Entities\Reforfil;
use \Suprimentos\Entities\Prazopgtaf;
use \Suprimentos\Entities\Impitmaf;
use \Suprimentos\Repositories\Reforeg;
use \Suprimentos\Repositories\Itemcole;
use \Suprimentos\Repositories\Qfoprodforn;
use \Financeiro\Logic\ContasPagar\ContasPrevistaAF;
use Geral\Entities\Parcmpsup;

use \Setforms2\Runtime\Helper\Procedure;
use \Setforms2\Runtime\Helper\Procedure\Param;
use \Geral\Util\DateUtil;
use \Geral\Util\Ambiente;

class AutorizacaoFornecimento
{

    public static function geraCodAutorizacaoFornecimento($cdfildest)
    {
        $codigo = Novocodigo::geraCodigo(
            \Geral\Util\Ambiente::getNrorg(),
            'AUTOFORNE'.$cdfildest.\Geral\Util\Ambiente::getCdFilial(),
            1
        );
        return $codigo;
    }


    public static function geraAfPorDataEntrega($nrinscjurforn, $cdfildest, $cdfillanc, $nrautoforn)
    {
        if (!empty($nrinscjurforn)) {
            /** @var $fornecedor \Geral\Entities\Fornecedor */
            $fornecedor = Fornecedor::getObjectByWhere("NRINSJURFORN = ? AND NRORG = ?", array($nrinscjurforn, \Geral\Util\Ambiente::getNrorg()));
            /* @var $itemautf \Geral\Entities\Itemautf */
            $itemautf = Itemautf::getObjectByWhere("NRORG = ? AND CDFILDEST = ? AND CDFILLANC = ? AND NRAUTOFORN = ?", array(\Geral\Util\Ambiente::getNrorg(), $cdfildest, $cdfillanc, $nrautoforn));
            if ($fornecedor->getIdgerafdtent() === 'S') {
                if (!empty($itemautf)) {
                    tk_disable_field('DTPROGENTAF');
                    return tk_get_context_vars('DTPROGENTAF', tk_get_active_window_name());
                } else {
                    tk_enable_field('DTPROGENTAF');
                    return now();
                }

            } else {
                tk_enable_field('DTPROGENTAF');
                if (!empty($itemautf)) {
                    return tk_get_context_vars('DTPROGENTAF', tk_get_active_window_name());
                } else {
                    return now();
                }
            }
        }
    }


    public static function aprovaAutorizacaoFornecimento($nrorg, $cdfildest, $cdfillanc, $nrautoforn)
    {
        try {
            /** @var  $itemautf \Geral\Entities\Itemautf */
            $itemautf = \Geral\Entities\Itemautf::getObjectsByWhere(
                "NRORG = ? AND CDFILDEST = ? AND CDFILLANC = ? AND NRAUTOFORN = ?",
                array($nrorg, $cdfildest, $cdfillanc, $nrautoforn));

            if(empty($itemautf) == false){
                foreach($itemautf as $item){
                    $novaDataEntrega = $item->getDtnovaentraf();
                    $novaQuantidade = $item->getQtnovaprogaf();

                    if(empty($novaDataEntrega) && empty($novaQuantidade)){
                        continue;
                    } else{
                        if(empty($novaDataEntrega) == false){
                            $item->setDtprogentaf($novaDataEntrega);
                            $item->setDtnovaentraf(null);
                        }
                        if(empty($novaQuantidade) == false){
                            $item->setQtprogentaf($novaQuantidade);
                            $item->setQtnovaprogaf(null);
                        }
                        $item->setDsjustitmaf(null);
                        $item->setDsjustitmafrpv(null);
                        $item->persist();
                    }
                }
            }


        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function aprovaAutorizacaoFornecimentoProduto(array $row, $nrorg){
        try{
            /** @var  $itemautf \Geral\Entities\Itemautf */
            $itemautf = \Geral\Entities\Itemautf::getObjectByWhere(
                "NRORG = ? AND CDFILDEST = ? AND CDFILLANC = ? AND NRAUTOFORN = ? AND CDMARCA = ? AND CDPRODUTO = ? AND DTPROGENTAF = ?",
                array($nrorg, $row['CDFILDEST'], $row['CDFILLANC'], $row['NRAUTOFORN'], $row['CDMARCA'], $row['CDPRODUTO'], $row['DTPROGENTAF']));

            if($itemautf instanceof Itemautf){
                $novaQuantidade = $row['QTNOVAPROGAF'];

                if(empty($row['DTNOVAENTRAF']) && empty($novaQuantidade)){
                    return;
                } else{
                    if(empty($row['DTNOVAENTRAF']) == false){
                        $novaDataEntrega = DateUtil::getDataDeString($row['DTNOVAENTRAF'],DateUtil::FORMATO_BRASILEIRO,true);
                        $itemautf->setDtprogentaf($novaDataEntrega);
                        $itemautf->setDtnovaentraf(null);
                    }
                    if(empty($novaQuantidade) == false){
                        $itemautf->setQtprogentaf($novaQuantidade);
                        $itemautf->setQtnovaprogaf(null);
                    }
                    $itemautf->setDsjustitmaf(null);
                    $itemautf->setDsjustitmafrpv(null);
                    $itemautf->persist();
                }
            }
        } catch (\Exception $e) {
            throw $e;
        }

    }

    public static function reprovaAutorizacaoFornecimento(array $row, $nrorg){
        try{
            /** @var  $itemautf \Geral\Entities\Itemautf */
            $itemautf = \Geral\Entities\Itemautf::getObjectByWhere(
                "NRORG = ? AND CDFILDEST = ? AND CDFILLANC = ? AND NRAUTOFORN = ? AND CDMARCA = ? AND CDPRODUTO = ?",
                array($nrorg, $row['CDFILDEST'], $row['CDFILLANC'], $row['NRAUTOFORN'], $row['CDMARCA'], $row['CDPRODUTO']));

            if(empty($row['DTNOVAENTRAF']) && empty($row['QTNOVAPROGAF'])){
                return;
            }

            $itemautf->setDtnovaentraf(null);
            $itemautf->setQtnovaprogaf(null);
            $itemautf->setDsjustitmafrpv($row['DSJUSTITMAFRPV']);
            $itemautf->setDsjustitmaf(null);

            $itemautf->persist();

        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function atualizaAprovacaoAf($nrorg, $cdfildest, $cdfillanc, $nrautoforn, $novoValorTotalAf){
        /** @var $autoforne \Planejamento\Entities\Autoforne */
        $autoforne = \Planejamento\Entities\Autoforne::getObjectByWhere(
            "NRORG = ? AND CDFILDEST = ? AND CDFILLANC = ? AND NRAUTOFORN = ?",
            array($nrorg, $cdfildest, $cdfillanc, $nrautoforn));

        $autoforne->setVrtotalaf($novoValorTotalAf);
        $autoforne->setVrtotalaforg($novoValorTotalAf);
        $autoforne->setIdsituatuaaf(null);
        $autoforne->setIdafforalim('N');
        $autoforne->persist();
        \Setforms2\Shared\Registry::getEntityManager()->flush();
    }


    public static function validaFornecedor($nrorg, $cdfilial, $cdforneced)
    {
        if (self::verificaAtendimentoFornecedorFilial($nrorg, $cdfilial, $cdforneced)) {
            tk_abort(tk_get_message_by_name("msg_cmp_fornecedor_nao_atende_filial"));
        }
        if (self::verificaAtendimentoFornecedorRegiao($nrorg, $cdfilial, $cdforneced)) {
            tk_abort(tk_get_message_by_name("msg_cmp_fornecedor_nao_atende_regiao"));
        }
    }

/*    public static function validaFornecedor($nrorg, $cdfilial, $cdforneced)
    {
        if (self::verificaAtendimentoFornecedorFilial($nrorg, $cdfilial, $cdforneced)) {
            \Compras\Logic\Fornecimento\Exception\FornecedorAF::fornecedorNaoAtendeFilial();
        }
        if (self::verificaAtendimentoFornecedorRegiao($nrorg, $cdfilial, $cdforneced)) {
            \Compras\Logic\Fornecimento\Exception\FornecedorAF::fornecedorNaoAtendeRegiao();
        }
    }*/


    public static function validaItensAutorizacaoFornecimento($nrorg, $cdfilial, $cdforneced, $cdproduto, $cdarvprod, $cdmarca, $data_autor_forn)
    {
        $prodMarca = self::consultaNomeProdutoMarca($nrorg, $cdproduto, $cdmarca);

        if (self::verificaProdutoInativo($nrorg, $cdfilial, $cdproduto)) {
            \Geral\Util\AcumuladorDeMensagens::getInstance()->addMensagem("msg_cmp_produto_inativo", array($prodMarca[0]), \Geral\Util\Mensagem::ERRO);
            return false;
        } elseif (self::verificaMarcaVinculadaProduto($nrorg, $cdarvprod, $cdmarca)) {
            \Geral\Util\AcumuladorDeMensagens::getInstance()->addMensagem("msg_cmp_marca_sem_vinculo_produto", array($prodMarca[0], $prodMarca[1]), \Geral\Util\Mensagem::ERRO);
            return false;
        } elseif (self::verificaFornecedorVinculadoProduto($nrorg, $cdforneced, $cdarvprod)) {
            \Geral\Util\AcumuladorDeMensagens::getInstance()->addMensagem("msg_cmp_fornecedor_sem_vinculo_produto", array($prodMarca[0]), \Geral\Util\Mensagem::ERRO);
            return false;
        } elseif (self::verificaMarcaQualificacaoVencida($nrorg, $cdforneced, $cdproduto, $cdmarca, $data_autor_forn)) {
            \Geral\Util\AcumuladorDeMensagens::getInstance()->addMensagem("msg_cmp_marca_qualificacao_vencida", array($prodMarca[0], $prodMarca[1]), \Geral\Util\Mensagem::ERRO);
            return false;
        } elseif (self::verificaProdutoNaoQualificadoFornecedor($nrorg, $cdfilial, $cdproduto, $cdforneced, $data_autor_forn)) {
            \Geral\Util\AcumuladorDeMensagens::getInstance()->addMensagem("msg_cmp_produto_nao_qualificado_fornecedor", array($prodMarca[0]), \Geral\Util\Mensagem::ERRO);
            return false;
        } else {
            return true;
        }

    }


    protected static function verificaProdutoInativo($nrorg, $cdfilial, $cdproduto)
    {
        /** @var $obj_Produto Produto */
        $obj_Produto =  \Geral\Entities\Produto::getRepository();
        $produto_inativo = $obj_Produto->consultaProdutoInativo($nrorg, $cdfilial, $cdproduto);
        return !empty($produto_inativo);
    }


    protected static function verificaMarcaVinculadaProduto($nrorg, $cdproduto, $cdmarca)
    {
        /** @var $obj_Marca Marca */
        $obj_Marca = \Geral\Entities\Marca::getRepository();
        $marca_vinculada = $obj_Marca->consultaMarcaVinculadaProduto($nrorg, $cdproduto, $cdmarca);
        return empty($marca_vinculada);
    }


    /**
     * Verfica se o produto está cadastrado para o respectivo fornecedor
     * @static
     * @param $nrorg
     * @param $cdforneced
     * @param $cdproduto
     * @return bool
     */
    protected static function verificaFornecedorVinculadoProduto($nrorg, $cdforneced, $cdproduto)
    {
        /** @var $obj_Fornecedor \Geral\Repositories\Fornecedor */
        $obj_Fornecedor = \Geral\Entities\Fornecedor::getRepository();
        $fornec_vinculado = $obj_Fornecedor->consultaFornecedorVinculadoProduto($nrorg, $cdforneced, $cdproduto);
        if ($fornec_vinculado[0]['CONTA'] == 0) {
            return true;
        }
        return false;
    }

//    protected static function verificaFornecedorVinculadoProduto($nrorg, $cdforneced, $cdproduto)
//    {
//        /** @var $obj_Fornecedor \Geral\Repositories\Fornecedor */
//        $obj_Fornecedor = \Geral\Entities\Fornecedor::getRepository();
//        $fornec_vinculado = $obj_Fornecedor->consultaFornecedorVinculadoProduto($nrorg, $cdforneced, $cdproduto);
//        return empty($fornec_vinculado);
//    }

    protected static function verificaMarcaQualificacaoVencida($nrorg, $cdfilial, $cdproduto, $cdmarca, $data_autor_forn)
    {
        /** @var $obj_Marca Marca */
        $obj_Marca = \Geral\Entities\Marca::getRepository();
        $qualificacao_vencida = $obj_Marca->consultaQualificacaoVencida($nrorg, $cdfilial, $cdproduto, $cdmarca, $data_autor_forn);
        return !empty($qualificacao_vencida);
    }


    protected static function verificaProdutoNaoQualificadoFornecedor($nrorg, $cdfilial, $cdproduto, $cdforneced, $data_autor_forn)
    {
        /** @var $obj_Qfoprodforn Qfoprodforn */
        $obj_Qfoprodforn = \Suprimentos\Entities\Qfoprodforn::getRepository();
        $produto_nao_qualificado = $obj_Qfoprodforn->consultaProdutoNaoQualificadoFornecedor($nrorg, $cdfilial, $cdproduto, $cdforneced, $data_autor_forn);
        return !empty($produto_nao_qualificado);
    }


    protected static function verificaAtendimentoFornecedorFilial($nrorg, $cdfilial, $cdforneced)
    {
        $fornecedor_nao_atende = Reforfil::getObjectByWhere("NRORG = ? AND CDFILIAL = ? AND CDFORNECED = ? AND CDPERIENTR IS NULL", array($nrorg, $cdfilial, $cdforneced));
        return is_object($fornecedor_nao_atende);
    }


    protected static function verificaAtendimentoFornecedorRegiao($nrorg, $cdfilial, $cdforneced)
    {
        $fornecedor_nao_atende = Reforeg::consultaAtendimentoFornecedorRegiao($nrorg, $cdfilial, $cdforneced);
        return !empty($fornecedor_nao_atende);
    }


    /**
     * Executa procedure FECHAMENTO_AF
     *
     * @param $nrorg
     * @param $cdfildest
     * @param $cdfillanc
     * @param $nrautoforn
     * @throws \Exception
     */
    protected static function executaProcedureFechamentoAF($nrorg, $cdfildest, $cdfillanc, $nrautoforn) {
        try {
            $procedure = new Procedure("FECHAMENTO_AF");
            self::addParametrosFechamentoAF($procedure, $nrorg, $cdfildest, $cdfillanc, $nrautoforn);
            $procedure->call();
        } catch (\Exception $e) {
            throw $e;
        }
    }


    /**
     * Adiciona os parametros da procedure FECHAMENTO_AF.
     *
     * @param \Setforms2\Runtime\Helper\Procedure $procedure
     * @param $nrorg
     * @param $cdfildest
     * @param $cdfillanc
     * @param $nrautoforn
     */
    protected static function addParametrosFechamentoAF(
        Procedure $procedure, $nrorg, $cdfildest, $cdfillanc, $nrautoforn)
    {
//        $procedure->addParam(
//            new Param(
//                'P_NRORG',
//                Param::PARAM_INPUT,
//                $nrorg,
//                Param::PARAM_TYPE_INT
//            )
//        );
        $procedure->addParam(
            new Param(
                'P_CDFILDEST',
                Param::PARAM_INPUT,
                $cdfildest,
                Param::PARAM_TYPE_STR
            )
        );
        $procedure->addParam(
            new Param(
                'P_CDFILLANC',
                Param::PARAM_INPUT,
                $cdfillanc,
                Param::PARAM_TYPE_STR
            )
        );
        $procedure->addParam(
            new Param(
                'P_NRAUTOFORN',
                Param::PARAM_INPUT,
                $nrautoforn,
                Param::PARAM_TYPE_STR
            )
        );

    }


    /**
     * Executa procedure VERIFICA_INCONS_AFS
     *
     * @param $nrorg
     * @param $cdfildest
     * @param $cdfillanc
     * @param $nrautoforn
     * @throws \Exception
     */
    protected static function verificaInconsistenciaAF($nrorg, $cdfildest, $cdfillanc, $nrautoforn)
    {
        try {
            $procedure = new Procedure("VERIFICA_INCONS_AFS");
            self::addParametrosFechamentoAF($procedure, $nrorg, $cdfildest, $cdfillanc, $nrautoforn);
            $procedure->call();
        } catch (\Exception $e) {
            throw $e;
        }

    }


    /**
     * Adiciona os parametros da procedure VERIFICA_INCONS_AFS.
     *
     * @param \Setforms2\Runtime\Helper\Procedure $procedure
     * @param $nrorg
     * @param $cdfildest
     * @param $cdfillanc
     * @param $nrautoforn
     */
    protected static function addParametrosVerificaInconsistenciaAF(
        Procedure $procedure, $nrorg, $cdfildest, $cdfillanc, $nrautoforn)
    {
//        $procedure->addParam(
//            new Param(
//                'P_NRORG',
//                Param::PARAM_INPUT,
//                $nrorg,
//                Param::PARAM_TYPE_INT
//            )
//        );
        $procedure->addParam(
            new Param(
                'P_CDFILDEST',
                Param::PARAM_INPUT,
                $cdfildest,
                Param::PARAM_TYPE_STR
            )
        );
        $procedure->addParam(
            new Param(
                'P_CDFILLANC',
                Param::PARAM_INPUT,
                $cdfillanc,
                Param::PARAM_TYPE_STR
            )
        );
        $procedure->addParam(
            new Param(
                'P_NRAUTOFORN',
                Param::PARAM_INPUT,
                $nrautoforn,
                Param::PARAM_TYPE_STR
            )
        );

    }


    /**
     * Executa procedure IMPOSTO_ITEMAF
     *
     * @param $nrorg
     * @param $cdempresa
     * @param $cdincesta
     * @param $cdforneced
     * @param $cdfildest
     * @param $cdfillanc
     * @param $nrautoforn
     * @param $dtinivalid     * @throws \Exception
     */
    public static function executaProcedureImpostoItemAF($nrorg, $cdempresa, $cdincesta, $cdforneced, $cdfildest, $cdfillanc, $nrautoforn, $dtinivalid)
    {
        try {
            $procedure = new Procedure("IMPOSTO_ITEMAF");
            self::addParametrosImpostoItemAF($procedure, $nrorg, $cdempresa, $cdincesta, $cdforneced, $cdfildest, $cdfillanc, $nrautoforn, $dtinivalid);
            $procedure->call();
        } catch (\Exception $e) {
            throw $e;
        }

    }


    /**
     * Adiciona os parametros da procedure IMPOSTO_ITEMAF.
     *
     * @param \Setforms2\Runtime\Helper\Procedure $procedure
     * @param $nrorg
     * @param $cdempresa
     * @param $cdincesta
     * @param $cdforneced
     * @param $cdfildest
     * @param $cdfillanc
     * @param $nrautoforn
     * @param $dtinivalid
     */
    protected static function addParametrosImpostoItemAF(
        Procedure $procedure, $nrorg, $cdempresa, $cdincesta, $cdforneced, $cdfildest, $cdfillanc, $nrautoforn, $dtinivalid)
    {
//        $procedure->addParam(
//            new Param(
//                'P_NRORG',
//                Param::PARAM_INPUT,
//                $nrorg,
//                Param::PARAM_TYPE_INT
//            )
//        );

        $procedure->addParam(
            new Param(
                'P_EMPRESA',
                Param::PARAM_INPUT,
                $cdempresa,
                Param::PARAM_TYPE_STR
            )
        );
        $procedure->addParam(
            new Param(
                'P_INSCESTA',
                Param::PARAM_INPUT,
                $cdincesta,
                Param::PARAM_TYPE_STR
            )
        );$procedure->addParam(
            new Param(
                'P_FORNECEDOR',
                Param::PARAM_INPUT,
                $cdforneced,
                Param::PARAM_TYPE_STR
            )
        );$procedure->addParam(
            new Param(
                'P_FILDEST',
                Param::PARAM_INPUT,
                $cdfildest,
                Param::PARAM_TYPE_STR
            )
        );
        $procedure->addParam(
            new Param(
                'P_FILLANC',
                Param::PARAM_INPUT,
                $cdfillanc,
                Param::PARAM_TYPE_STR
            )
        );
        $procedure->addParam(
            new Param(
                'P_LANCTOAF',
                Param::PARAM_INPUT,
                $nrautoforn,
                Param::PARAM_TYPE_STR
            )
        );
        $procedure->addParam(
            new Param(
                'P_INIVALID',
                Param::PARAM_INPUT,
                $dtinivalid,
                Param::PARAM_TYPE_STR
            )
        );

    }


    /**
     * @return \Suprimentos\Entities\InconsAf
     */
    public static function consultaInconsAf ($nrorg, $cdfildest, $cdfillanc, $nrautoforn) {
        /* @var \Suprimentos\Entities\InconsAf $obj_Incons_af*/
        $obj_Incons_af = \Suprimentos\Entities\InconsAf::getObjectByWhere('NRORG = ? AND CDFILDEST = ? AND CDFILLANC = ? AND NRAUTOFORN = ?', array($nrorg, $cdfildest, $cdfillanc, $nrautoforn));
        if ($obj_Incons_af == false) {
            return false;
        }
        return true;
    }

    /**
     * @return \Suprimentos\Entities\InconsItaf
     */
    public static function consultaInconsItaF ($nrorg, $cdfildest, $cdfillanc, $nrautoforn) {
        /* @var \Suprimentos\Entities\InconsItaf $obj_Incons_itaf */
        $obj_Incons_itaf = \Suprimentos\Entities\InconsItaf::getObjectByWhere('NRORG = ? AND CDFILDEST = ? AND CDFILLANC = ? AND NRAUTOFORN = ?', array($nrorg, $cdfildest, $cdfillanc, $nrautoforn));
        if ($obj_Incons_itaf == false) {
            return false;
        }
        return true;
    }


    public static function atualizaSituacaoAf($nrorg, $cdfildest, $cdfillanc, $nrautoforn)
    {
        $incons_af = self::consultaInconsAf($nrorg, $cdfildest, $cdfillanc, $nrautoforn);
        $incons_itaf = self::consultaInconsItaF($nrorg, $cdfildest, $cdfillanc, $nrautoforn);
        $situacao = null;
        if ($incons_af == true || $incons_itaf == true) {
            // Seta a situação da AF para inconsistente
            $situacao = 1;
            tk_set_field_value('IDSITUATUAAF', $situacao, 'tab_cmp03900_autoforne', 'cmp03900');
            tk_warning(tk_get_message_by_name('msg_cmp03900_incosistencia_af'));
        } else {
            // Seta a situação da AF para consistente
            $situacao = 0;
            tk_set_field_value('IDSITUATUAAF', $situacao, 'tab_cmp03900_autoforne', 'cmp03900');
        }
        return $situacao;
    }


    public static function alteraSituacaoAF($nrorg, $cdfildest, $cdfillanc, $nrautoforn, $situacao)
    {
        try {
            $connection = \Setforms2\Shared\Registry::getEntityManager();
            /* @var \Planejamento\Entities\Autoforne $obj_Autoforne */
            $obj_Autoforne = Autoforne::getObjectByWhere(
                "NRORG = ? AND CDFILDEST = ? AND CDFILLANC = ? AND NRAUTOFORN = ?",
                array($nrorg, $cdfildest, $cdfillanc, $nrautoforn)
            );
            $obj_Autoforne->setIdsituatuaaf($situacao);
            $obj_Autoforne->persist();
            $connection->flush();
        } catch (\Exception $e) {
            tk_warning($e->getMessage());
        }
    }


    public static function executaFechamentoAF($nrorg, $cdfildest, $cdfillanc, $nrautoforn, $id_baixa_af, $id_forma_pgto, $cdempresa, $cdincesta, $cdforneced, $dtinivalid)
    {
        try {
            //Fechamento so é feito para AF's em aberto.
            if ($id_baixa_af == 0) {
                self::verificaInconsistenciaAF($nrorg, $cdfildest, $cdfillanc, $nrautoforn);
                $windowName = tk_get_active_window_name();
                if($windowName != 'win_cmp07604_000' && $windowName != 'cmp07600_aprovacao_af'){
                    self::atualizaSituacaoAf($nrorg, $cdfildest, $cdfillanc, $nrautoforn);
                }
                self::executaProcedureImpostoItemAF($nrorg, $cdempresa, $cdincesta, $cdforneced, $cdfildest, $cdfillanc, $nrautoforn, $dtinivalid);
                if ($id_forma_pgto == 'V') {
                    $obj_Prazopgtaf = Prazopgtaf::getObjectByWhere(
                        "NRORG = ? AND CDFILDEST = ? AND CDFILLANC = ? AND NRAUTOFORN = ?",
                        array($nrorg, $cdfildest, $cdfillanc, $nrautoforn));

                    if (!is_object($obj_Prazopgtaf)) {
                        \Geral\Util\AcumuladorDeMensagens::getInstance()->addMensagem(
                            "msg_cmp_af_sem_prazo_pagamento", array($nrautoforn), \Geral\Util\Mensagem::ALERTA
                        );
                    } else {
                        self::excluiContasPagarPrevista($nrorg, $cdfildest, $cdfillanc, $nrautoforn);
                        ContasPrevistaAF::executaGeraContasPagarPrevista($nrorg, $cdfildest, $cdfillanc, $nrautoforn);
                        self::executaProcedureFechamentoAF($nrorg, $cdfildest, $cdfillanc, $nrautoforn);
                        \Geral\Util\AcumuladorDeMensagens::getInstance()->addMensagem(
                            "msg_cmp_fechamento_af_realizado", array($nrautoforn), \Geral\Util\Mensagem::VALIDACAO
                        );
                    }
                } else {
                    self::excluiContasPagarPrevista($nrorg, $cdfildest, $cdfillanc, $nrautoforn);
                    ContasPrevistaAF::executaGeraContasPagarPrevista($nrorg, $cdfildest, $cdfillanc, $nrautoforn);
                    self::executaProcedureFechamentoAF($nrorg, $cdfildest, $cdfillanc, $nrautoforn);

                    \Geral\Util\AcumuladorDeMensagens::getInstance()->addMensagem(
                        'msg_cmp_fechamento_af_realizado', array($nrautoforn), \Geral\Util\Mensagem::VALIDACAO
                    );
                }
            } else {
                \Geral\Util\AcumuladorDeMensagens::getInstance()->addMensagem(
                    "msg_cmp_fechamento_af_em_aberto", array($nrautoforn), \Geral\Util\Mensagem::ALERTA
                );
            }
            self::atualizaVrtotalaf($nrorg, $cdfildest, $cdfillanc, $nrautoforn);
//            tk_reload_data_with_filter('tab_cmp03900_autoforne', tk_get_active_window_name());
        } catch (\Exception $e) {
            throw $e;
        }
    }


    public static function atualizaVrtotalaf($nrorg, $cdfildest, $cdfillanc, $nrautoforn)
    {
        try {
            $total = 0;
            /* @var \Geral\Entities\Itemautf $objs_Itemautf */
            $objs_Itemautf = Itemautf::getObjectsByWhere(
                "NRORG = ? AND CDFILDEST = ? AND CDFILLANC = ? AND NRAUTOFORN = ?",
                array($nrorg, $cdfildest, $cdfillanc, $nrautoforn)
            );
            if (!empty($objs_Itemautf)) {
                /* @var \Geral\Entities\Itemautf $obj */
                foreach ($objs_Itemautf as $chave => $obj) {
                    $total += $obj->getQtprogentaf() * $obj->getVrpreuniaf();
                }
            }

            tk_set_field_value('VRTOTALAF', $total, 'tab_cmp03900_autoforne', tk_get_active_window_name());
            tk_set_context_vars('VRTOTALAF',tk_get_active_window_name(),$total);

            $connection = \Setforms2\Shared\Registry::getEntityManager();
            /* @var \Planejamento\Entities\Autoforne $obj_Autoforne */
            $obj_Autoforne = Autoforne::getObjectByWhere(
                "NRORG = ? AND CDFILDEST = ? AND CDFILLANC = ? AND NRAUTOFORN = ?",
                array($nrorg, $cdfildest, $cdfillanc, $nrautoforn)
            );
            $obj_Autoforne->setVrtotalaf($total);
            $obj_Autoforne->persist();
            $connection->flush();
        } catch (\Exception $e) {
            tk_warning($e->getMessage());
        }
    }


    public static function mudaStatusAF($nrorg, $cdfildest, $cdfillanc, $nrautoforn, $id_baixa_af, $id_novo_status, $cdoperador){
        try {
            if ($id_baixa_af != 0 && $id_baixa_af != 1 && $id_baixa_af != 2 && $id_novo_status != 0) {
                \Geral\Util\AcumuladorDeMensagens::getInstance()->addMensagem("msg_muda_status_af_em_aberto", array($nrautoforn));
            } else if ($id_baixa_af == 1 || $id_baixa_af == 2) {
                \Geral\Util\AcumuladorDeMensagens::getInstance()->addMensagem("msg_cmp_status_baixa_total_parcial", array($nrautoforn));
            } else {

                if ($id_novo_status == 3 || $id_novo_status == 4 || $id_novo_status == 5) { //Status BAIXA MANUAL ou ENCERRADA ou CANCELADA
                    self::excluiContasPagarPrevista($nrorg, $cdfildest, $cdfillanc, $nrautoforn);
                }

                /** @var $obj_Autoforne Autoforne */
                $obj_Autoforne = Autoforne::getObjectByWhere(
                    "NRORG = ? AND CDFILDEST = ? AND CDFILLANC = ? AND NRAUTOFORN = ?",
                    array($nrorg, $cdfildest, $cdfillanc, $nrautoforn));
                $id_novo_status == 0 ? $obj_Autoforne->setIdbaixaaf(null) : $obj_Autoforne->setIdbaixaaf($id_novo_status);
                $obj_Autoforne->persist();

                if ($id_novo_status == 3) { //Status BAIXA MANUAL
                    $objs_Itemautf = Itemautf::getObjectsByWhere(
                        "NRORG = ? AND CDFILDEST = ? AND CDFILLANC = ? AND NRAUTOFORN = ?", array($nrorg, $cdfildest, $cdfillanc, $nrautoforn));
                    /** @var $obj_Itemautf Itemautf */
                    foreach ($objs_Itemautf as $obj_Itemautf) {
                        $obj_Itemautf->setIdbaixitmaf(3);
                        $obj_Itemautf->setCdoperbaixa($cdoperador);
                        $obj_Itemautf->setDtbaixitmaf(\Geral\Util\DateUtil::getDataAtual());
                        $obj_Itemautf->persist();
                    }
                }
                if ($id_novo_status == 0) { //Status EM ABERTO
                    $objs_Itemautf = Itemautf::getObjectsByWhere(
                        "NRORG = ? AND CDFILDEST = ? AND CDFILLANC = ? AND NRAUTOFORN = ?", array($nrorg, $cdfildest, $cdfillanc, $nrautoforn));
                    /** @var $obj_Itemautf Itemautf */
                    foreach ($objs_Itemautf as $obj_Itemautf) {
                        $obj_Itemautf->setIdbaixitmaf(null);
                        $obj_Itemautf->setCdoperbaixa(null);
                        $obj_Itemautf->setDtbaixitmaf(null);
                        $obj_Itemautf->persist();
                    }
                }
                \Setforms2\Shared\Registry::getEntityManager()->flush();

//                \Geral\Util\AcumuladorDeMensagens::getInstance()->addMensagem("msg_muda_status_af_ok", array($nrautoforn));
            }

        } catch (\Exception $e) {
            throw $e;
        }
    }


    public static function excluiContasPagarPrevista($nrorg, $cdfildest, $cdfillanc, $nrautoforn)
    {
        ContasPrevistaAF::executaExcluiContasPagarPrevista($nrorg, $cdfildest, $cdfillanc, $nrautoforn);
    }


    public static function validaPercentualFormaPgto($nrorg, $cdfildest, $cdfillanc, $nrautoforn)
    {
        // Pega as linhas que ainda não foram salvas e coloca os campos prazo e percenutal no array $vet_linhas
        $linhas = Framework::getInstance()->getDataset();
        $vet_linhas = array();
        $percentual_total = 0;

        foreach ($linhas as $key => $key2) {
            $aux = array();
            $aux['NRDIASPGTOAF'] = $key2['NRDIASPGTOAF'];
            $aux['VRPERCPGTOAF'] = $key2['VRPERCPGTOAF'];
            $vet_linhas[] = $aux;
            $percentual_total += $key2['VRPERCPGTOAF'];
        }

        // Pega as linhas que foram salvas no banco
        $linhas_banco = self::retornaPrazoPercent($nrorg, $cdfildest, $cdfillanc, $nrautoforn);
        if (!empty($linhas_banco)) {
            $prazo_igual = array();
            /* @var $key2 Prazopgtaf*/
            foreach ($linhas_banco as $key => $key2) {
                // Verifica se o prazo de alguma linha do banco é igual ao prazo de alguma linha nova ou alterada
                foreach ($vet_linhas as $key_linhas => $key2_linhas) {
                    // Caso seja igual, adiciona no array $prazo_igual
                    if ($key2->getNrdiaspgtoaf() == $key2_linhas['NRDIASPGTOAF']) {
                        $prazo_igual[] = $key2->getNrdiaspgtoaf();
                    }
                }
            }
            // Percorre as linhas do banco. Se o prazo da linha atual for diferente de algum valor do array $prazo_igual, adiciona percentual em $percentual_total
            foreach ($linhas_banco as $key => $key2) {
                if (!empty($prazo_igual)) {
                    $aux = array();
                    $aux[] = $key2->getNrdiaspgtoaf();
                    $intersecao = array_intersect($aux, $prazo_igual);
                    if (empty($intersecao)) {
                        $percentual_total += $key2->getVrpercpgtoaf();
                    }
                } else {
                    $percentual_total += $key2->getVrpercpgtoaf();
                }
            }
        }

        if ($percentual_total != 100) {
            tk_abort(\Compras\Logic\Fornecimento\Exception\FormaPagamento::excedeSomatorioPercentual());
        } else {
            foreach ($linhas as $key => $key2) {
                $nrdiaspgtoaf = $key2['NRDIASPGTOAF'];
                $vrpercpgtoaf = $key2['VRPERCPGTOAF'];
                $dtinclusao  = DateUtil::getDataAtual();
                if ($key2['_STATE'] === 'I') {
                    Prazopgtaf::inserePrazoPercentual($nrdiaspgtoaf, $vrpercpgtoaf, $cdfildest, $cdfillanc, $nrautoforn, $nrorg, $dtinclusao);
                } else {
                    Prazopgtaf::atualizaPrazoPercentual($nrdiaspgtoaf, $vrpercpgtoaf, $cdfildest, $cdfillanc, $nrautoforn, $nrorg, $dtinclusao);
                }
            }
            tk_reload_data_with_filter('tab_cmp03900_prazo_percent', 'cmp03900');
            tk_message(tk_get_message_by_name('msg_gep_Atualizado'), 'success');
        }
    }


    public static function validaPrazoFormaPgto($nrorg, $cdfildest, $cdfillanc, $nrautoforn)
    {
        // Pega as linhas inseridas
        $linhas = Framework::getInstance()->getDataset();
        $vet_linhas = array();
        $igual = false;
        $dias = "";

        foreach ($linhas as $key => $key2) {
            $aux = array();
            $aux['_ID'] = $key2['_ID'];
            $aux['_STATE'] = $key2['_STATE'];
            $aux['NRDIASPGTOAF'] = $key2['NRDIASPGTOAF'];
            $vet_linhas[] = $aux;
        }
        $vet_linhas_2 = $vet_linhas;
        // Verifica se entre as linhas inseridas existe prazos iguais
        foreach ($vet_linhas as $key => $key2) {
            foreach ($vet_linhas_2 as $key_linhas_2 => $key2_linhas_2) {
                $intersecao = array_intersect_assoc($key2, $key2_linhas_2);
                if ($key2['_ID'] != $key2_linhas_2['_ID'] AND array_key_exists('NRDIASPGTOAF', $intersecao)) {
                    $igual = true;
                }
            }
        }
        if ($igual) {
            tk_abort(\Compras\Logic\Fornecimento\Exception\FormaPagamento::prazoDiasExistente());
        }

        // Verifica alguma das linhas inseridas já está no banco
        foreach ($linhas as $key2) {
            if ($key2['_STATE'] === "I") {
                $dias .= $key2['NRDIASPGTOAF'].'_';
            }
        }
        $linhas_banco = self::verificaPrazo($nrorg, $cdfildest, $cdfillanc, $nrautoforn, substr($dias, 0, -1));
        if (empty($linhas_banco[0]) === false) {
            tk_abort(\Compras\Logic\Fornecimento\Exception\FormaPagamento::prazoDiasExistente());
        }
    }


    public static function limpaCamposFormaPgto($nrorg, $cdfildest, $cdfillanc, $nrautoforn, $forma_pgto)
    {
        $connection = \Setforms2\Shared\Registry::getEntityManager();
        $connection->beginTransaction();
        try {
            /** @var $obj_Autoforne Autoforne */
            $obj_Autoforne = Autoforne::getObjectByWhere(
                "NRORG = ? AND CDFILDEST = ? AND CDFILLANC = ? AND NRAUTOFORN = ?",
                array($nrorg, $cdfildest, $cdfillanc, $nrautoforn));

            /** @var $objs_Prazopgtaf Prazopgtaf */
            $objs_Prazopgtaf = Prazopgtaf::getObjectsByWhere(
                "NRORG = ? AND CDFILDEST = ? AND CDFILLANC = ? AND NRAUTOFORN = ?",
                array($nrorg, $cdfildest, $cdfillanc, $nrautoforn)
            );

            switch ($forma_pgto) {
                case 'null':
                    $obj_Autoforne->setNrparcpgtoaf(null);
                    $obj_Autoforne->setNrdiapgfixaf(null);
                    $obj_Autoforne->setIdutdiapgtaf(null);
                    $obj_Autoforne->setCdformpgto(null);
                    $obj_Autoforne->persist();
                    $connection->flush();

                    /** @var $obj_Prazopgtaf Prazopgtaf */
                    foreach ($objs_Prazopgtaf as $obj_Prazopgtaf) {
                        $obj_Prazopgtaf->delete();
                        $connection->flush();
                    }

                    break;
                case 'F':
                    $obj_Autoforne->setCdformpgto(null);
                    $obj_Autoforne->persist();
                    $connection->flush();

                    /** @var $obj_Prazopgtaf Prazopgtaf */
                    foreach ($objs_Prazopgtaf as $obj_Prazopgtaf) {
                        $obj_Prazopgtaf->delete();
                        $connection->flush();
                    }

                    break;

                case 'V':
                    $obj_Autoforne->setNrparcpgtoaf(null);
                    $obj_Autoforne->setNrdiapgfixaf(null);
                    $obj_Autoforne->setIdutdiapgtaf(null);
                    $obj_Autoforne->persist();
                    $connection->flush();

                    break;
            }

            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }

    }


    public static function identificaComprador($nrorg, $cdoperador)
    {
        $obj_Comprador = Comprador::getObjectByWhere("NRORG = ? AND CDOPERADOR = ?", array($nrorg, $cdoperador));
        if (is_object($obj_Comprador)) return $cdoperador;  return '';
    }


    public static function verificaItemAFRepetido($nrorg, $cdfildest, $cdfillanc, $nrautoforn, $dtprogentaf, $cdproduto, $cdmarca, $valor, $preco, $id_novo_item)
    {

        if (DateUtil::dataPertenceAoFormato($dtprogentaf, DateUtil::FORMATO_BRASILEIRO_DATAHORA)) {
            $dtprogentaf = DateUtil::getDataDeString($dtprogentaf, DateUtil::FORMATO_BRASILEIRO_DATAHORA);
        } else {
            $dtprogentaf = DateUtil::truncateData(DateUtil::getDataDeString($dtprogentaf));
        }

        try {
            /* @var $obj_ItemAutf Itemautf*/
            $obj_ItemAutf = Itemautf::getObjectByWhere(
                "NRORG = ? AND CDFILDEST = ? AND CDFILLANC = ? AND DTPROGENTAF = ? AND NRAUTOFORN = ? AND CDPRODUTO = ? AND CDMARCA = ?",
                array($nrorg, $cdfildest, $cdfillanc, $dtprogentaf, $nrautoforn, $cdproduto, $cdmarca));
            if (is_object($obj_ItemAutf)) {
                if ($id_novo_item == 'U') {
                    $connection = \Setforms2\Shared\Registry::getEntityManager();
                    $connection->beginTransaction();

                    $obj_ItemAutf->setQtprogentaf($valor);
                    $obj_ItemAutf->setVrpreuniaf($preco);

                    $obj_ItemAutf->persist();
                    $connection->flush();
                    $connection->commit();

                    $prodMarca = self::consultaNomeProdutoMarca($nrorg, $cdproduto, $cdmarca);
                    \Geral\Util\AcumuladorDeMensagens::getInstance()->addMensagem(
                        "msg_cmp_alterado", array($prodMarca[0], $prodMarca[1]), \Geral\Util\Mensagem::VALIDACAO
                    );

                    return true;

                } else {
                    if($id_novo_item == 'I'){
                        $prodMarca = self::consultaNomeProdutoMarca($nrorg, $cdproduto, $cdmarca);
                        \Geral\Util\AcumuladorDeMensagens::getInstance()->addMensagem(
                            "msg_cmp_produto_marca_item_af_existente", array($prodMarca[0], $prodMarca[1]), \Geral\Util\Mensagem::ERRO
                        );
                    }
                    return false;
                }

            } else {
                $connection = \Setforms2\Shared\Registry::getEntityManager();
                $connection->beginTransaction();

                $obj_ItemAutf = new Itemautf();
                $obj_ItemAutf->setNrorg($nrorg);
                $obj_ItemAutf->setCdfildest($cdfildest);
                $obj_ItemAutf->setCdfillanc($cdfillanc);
                $obj_ItemAutf->setNrautoforn($nrautoforn);
                $obj_ItemAutf->setDtprogentaf($dtprogentaf);
                $obj_ItemAutf->setCdproduto($cdproduto);
                $obj_ItemAutf->setCdmarca($cdmarca);
                $obj_ItemAutf->setQtprogentaf($valor);
                $obj_ItemAutf->setVrpreuniaf($preco);
                $obj_ItemAutf->setNrparcitmaf('00');
                $obj_ItemAutf->setIdbaixitmfutura('X');
                $obj_ItemAutf->setIddevolucaonfit('N');

                $obj_ItemAutf->persist();

                $connection->flush();
                $connection->commit();

                $prodMarca = self::consultaNomeProdutoMarca($nrorg, $cdproduto, $cdmarca);
                \Geral\Util\AcumuladorDeMensagens::getInstance()->addMensagem("msg_cmp_salvo", array($prodMarca[0], $prodMarca[1]), \Geral\Util\Mensagem::VALIDACAO);
                return true;
            }

        } catch (\Exception $e) {
            $connection->rollback();
            \Geral\Util\TratamentoDeMensagens::exibirMensagens($e);
        }

    }


    public static function verificaValorMaiorZero($campo, $valor, $nrorg, $cdproduto, $cdmarca)
    {
        $prodMarca = self::consultaNomeProdutoMarca($nrorg, $cdproduto, $cdmarca);

        if ($campo == 'QTPROGENTAF' && (float)$valor <= 0.000) {
            \Geral\Util\AcumuladorDeMensagens::getInstance()->addMensagem("msg_cmp_quantidade_maior_zero", array($prodMarca[0], $prodMarca[1]), \Geral\Util\Mensagem::ERRO);
            return false;
        } elseif ($campo == 'VRPREUNIAF' && (float)$valor <= 0.000) {
            \Geral\Util\AcumuladorDeMensagens::getInstance()->addMensagem("msg_cmp_preco_maior_zero", array($prodMarca[0], $prodMarca[1]), \Geral\Util\Mensagem::ERRO);
            return false;
        } else {
            return true;
        }

    }

    public static function consultaMelhorPreco($nrorg, $cdfildest, $dtprogentaf, $cdforneced, $cdarvprod)
    {
        /** @var $obj_Itemcole Itemcole */
        $obj_Itemcole = \Suprimentos\Entities\Itemcole::getRepository();
        $consulta_preco = $obj_Itemcole->consultaMelhorPreco($nrorg, $cdfildest, $dtprogentaf, $cdforneced, $cdarvprod);
        $melhor_preco = array(
            'CDMARCA' => isset($consulta_preco[0]['CDMARCA'])      ? $consulta_preco[0]['CDMARCA']      : null,
            'NMMARCA' => isset($consulta_preco[0]['NMMARCA'])      ? $consulta_preco[0]['NMMARCA']      : null,
            'PRECO'   => isset($consulta_preco[0]['VRPRECUNIITM']) ? $consulta_preco[0]['VRPRECUNIITM'] : null,
        );
        return $melhor_preco;
    }


    /**
     * Insere a Forma de Pagamento do Fornecedor na AF
     * @static
     * @param integer $nrorg      Numero da organizacao
     * @param integer $cdfildest  Codigo da filial de destino
     * @param integer $cdfillanc  Codigo da filial de lancamento
     * @param integer $nrautoforn Codigo da AF
     *
     * @throws \Exception
     */
    public static function insereFormaPagamento($nrorg, $cdfildest, $cdfillanc, $nrautoforn)
    {
        $connection = \Setforms2\Shared\Registry::getEntityManager();

        try {
            $connection->beginTransaction();

            /** @var $obj_Autoforne Autoforne */
            $obj_Autoforne = Autoforne::getObjectByWhere("NRORG = ? AND CDFILDEST = ? AND CDFILLANC = ? AND NRAUTOFORN = ?",
                array($nrorg, $cdfildest, $cdfillanc, $nrautoforn));

            //Consulta dados da forma de pagamento da tabela Fornecedor
            /** @var $obj_Fornecedor Fornecedor */
            $obj_Fornecedor = Fornecedor::getObjectByWhere("NRORG = ? AND CDFORNECED = ?", array($nrorg, $obj_Autoforne->getCdforneced()));

            $obj_Autoforne->setCdformpgto($obj_Fornecedor->getCdformpgto());
            $obj_Autoforne->setIdtpforpgtaf($obj_Fornecedor->getIdtpforpgtfo());
            $obj_Autoforne->setNrparcpgtoaf($obj_Fornecedor->getNrparcpgtofo());
            $obj_Autoforne->setNrdiapgfixaf($obj_Fornecedor->getNrdiapgfixfo());
            $obj_Autoforne->setIdutdiapgtaf($obj_Fornecedor->getIdutdiapgtfo());

            $obj_Autoforne->persist();

            if ($obj_Fornecedor->getIdtpforpgtfo() == 'V') {

                //Consulta dados da forma de pagamento da tabela Prazopgtforn
                /** @var $obj_Prazopgtforn Prazopgtforn */
                $obj_Prazopgtforn = Prazopgtforn::getObjectByWhere("NRORG = ? AND CDFORNECED = ?", array($nrorg, $obj_Autoforne->getCdforneced()));

                /** @var $obj_Prazopgtaf Prazopgtaf */
                $obj_Prazopgtaf = new Prazopgtaf();
                $obj_Prazopgtaf->setNrorg($nrorg);
                $obj_Prazopgtaf->setCdfildest($cdfildest);
                $obj_Prazopgtaf->setCdfillanc($cdfillanc);
                $obj_Prazopgtaf->setNrautoforn($nrautoforn);
                $obj_Prazopgtaf->setNrdiaspgtoaf($obj_Prazopgtforn->getNrdiaspgtofo());
                $obj_Prazopgtaf->setVrpercpgtoaf($obj_Prazopgtforn->getVrpercpgtofo());
                $obj_Prazopgtaf->setDtinclusao(DateUtil::getDataAtual());
                $obj_Prazopgtaf->setDtultatu(DateUtil::getDataAtual());
                $obj_Prazopgtaf->setCdoperinclusao(Ambiente::getCdoperador());
                $obj_Prazopgtaf->setCdoperultatu(Ambiente::getCdoperador());

                $obj_Prazopgtaf->persist();
            }

            $connection->flush();
            $connection->commit();

        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }

    }


    /**
     * @static
     * Provisorio
     */
    public static function buscaCodigoFornecedor($nrorg, $nrinscforn)
    {
        /** @var $obj_Fornecedor Fornecedor */
        $obj_Fornecedor = Fornecedor::getObjectByWhere("NRORG = ? AND NRINSJURFORN = ?", array($nrorg, $nrinscforn));
        return $obj_Fornecedor->getCdforneced();

    }


    /**
     * Converte a data de String para Objeto
     * @static
     * @param String
     */
    public static function converteData($data)
    {
        if (!empty($data) && is_string($data)) {
            if (DateUtil::dataPertenceAoFormato($data, DateUtil::FORMATO_BRASILEIRO_DATAHORA)) {
                $$data = DateUtil::truncateData(DateUtil::truncateString($data));
            }
            $data = DateUtil::truncateData(DateUtil::getDataDeString($data));
            return $data;
        }
        return null;
    }

    /**
     * Funçao criada  porque nao sera necessario editar produto e marca
     * com isso a versao browser precisa ser refeita com os campos produto e marca desabilitados depois de inserir
     * e a funcao verificaItemAFRepetido() ser substiuida por essa
     * @static
     * @return bool
     */
    public static function salvaProduto( $state,
        $dtbaixitmaf, $idmotprescaf, $idtipolanitm, $cdmarcaol, $qtsaidoplog, $idbaixitafs, $qtefafsaidol, $idgerprafsol,
        $idestoqcons, $dtultconsest, $qtprogentori, $cdfildestol, $cdfillancol, $nrautofornol, $cdfilcotitaf, $nrcotpreitaf,
        $cdoperbaixa, $cdmarcotitaf, $cdregcotitaf, $cdforcotitaf, $cdprocotitaf, $nrlistaitaf, $cdoperaprov, $qtnovaprogaf,
        $dtnovaentraf, $dsjustitmaf, $dthraprovaf, $nrorg, $dtinclusao, $dtultatu, $idtplaniteaf, $cdfilagrupaf, $nragrupaf,
        $cdfildest, $cdfillanc, $nrautoforn, $dtprogentaf, $cdproduto, $cdmarca, $cdforneced, $cdsegmarca, $nrparcitmaf,
        $dtefetentaf, $qtprogentaf, $vrpreuniaf, $vrperdesaf, $qtefetentraf, $cdempresa, $idtipemitmaf, $vrprececoaf,
        $vrpreccotaf, $idbaixitmaf, $nrlanctonf, $txobsitmaf, $vrdescitmaf, $nrloteestq, $nrsublote, $cdprodprodaf,
        $dtentritmsol, $dsinconsitaf, $idmotprescap, $qtvoluitemaf)
    {


        try {
            // Insere no banco
            if ($state === 'I' ) {
                // Converte datas de String para Objeto
                $dtbaixitmaf = self::converteData($dtbaixitmaf);
                $dtultconsest = self::converteData($dtultconsest);
                $dtnovaentraf = self::converteData($dtnovaentraf);
                $dthraprovaf = self::converteData($dthraprovaf);
                $dtinclusao = DateUtil::getDataDeString($dtinclusao, DateUtil::FORMATO_BRASILEIRO_DATAHORA);
                $dtultatu = DateUtil::getDataDeString($dtultatu, DateUtil::FORMATO_BRASILEIRO_DATAHORA);
                $dtprogentaf = self::converteData($dtprogentaf);
                $dtefetentaf = self::converteData($dtefetentaf);
                $dtentritmsol = self::converteData($dtentritmsol);

                if (self::verificaItemRepetido($nrorg, $cdfildest, $cdfillanc, $nrautoforn, $dtprogentaf, $cdproduto, $cdmarca) ==  false) {
                    /* @var $obj_ItemAutf Itemautf*/
//                    $connection = \Setforms2\Shared\Registry::getEntityManager();
//                    $connection->beginTransaction();
                    /** @var $obj \Geral\Entities\Itemautf*/
                    $obj = new \Geral\Entities\Itemautf();

                    $obj->setDtbaixitmaf($dtbaixitmaf);
                    $obj->setIdmotprescaf($idmotprescaf);
                    $obj->setIdtipolanitm($idtipolanitm);
                    $obj->setCdmarcaol($cdmarcaol);
                    $obj->setQtsaidoplog($qtsaidoplog);
                    $obj->setIdbaixitafs($idbaixitafs);
                    $obj->setQtefafsaidol($qtefafsaidol);
                    $obj->setIdgerprafsol($idgerprafsol);
                    $obj->setIdestoqcons($idestoqcons);
                    $obj->setDtultconsest($dtultconsest);
                    $obj->setQtprogentori($qtprogentori);
                    $obj->setCdfildestol($cdfildestol);
                    $obj->setCdfillancol($cdfillancol);
                    $obj->setNrautofornol($nrautofornol);
                    $obj->setCdfilcotitaf($cdfilcotitaf);
                    $obj->setNrcotpreitaf($nrcotpreitaf);
                    $obj->setCdoperbaixa($cdoperbaixa);
                    $obj->setCdmarcotitaf($cdmarcotitaf);
                    $obj->setCdregcotitaf($cdregcotitaf);
                    $obj->setCdforcotitaf($cdforcotitaf);
                    $obj->setCdprocotitaf($cdprocotitaf);
                    $obj->setNrlistaitaf($nrlistaitaf);
                    $obj->setCdoperaprov($cdoperaprov);
                    $obj->setQtnovaprogaf($qtnovaprogaf);
                    $obj->setDtnovaentraf($dtnovaentraf);
                    $obj->setDsjustitmaf($dsjustitmaf);
                    $obj->setDthraprovaf($dthraprovaf);
                    $obj->setNrorg($nrorg);
                    $obj->setDtinclusao($dtinclusao);
                    $obj->setDtultatu($dtultatu);
                    $obj->setIdtplaniteaf($idtplaniteaf);
                    $obj->setCdfilagrupaf($cdfilagrupaf);
                    $obj->setNragrupaf($nragrupaf);
                    $obj->setCdfildest($cdfildest);
                    $obj->setCdfillanc($cdfillanc);
                    $obj->setNrautoforn($nrautoforn);
                    $obj->setDtprogentaf($dtprogentaf);
                    $obj->setCdproduto($cdproduto);
                    $obj->setCdmarca($cdmarca);
                    $obj->setCdforneced($cdforneced);
                    $obj->setCdsegmarca($cdsegmarca);
                    $obj->setNrparcitmaf($nrparcitmaf);
                    $obj->setDtefetentaf($dtefetentaf);
                    $obj->setQtprogentaf($qtprogentaf);
                    $obj->setVrpreuniaf($vrpreuniaf);
                    $obj->setVrperdesaf($vrperdesaf);
                    $obj->setQtefetentraf($qtefetentraf);
                    $obj->setCdempresa($cdempresa);
                    $obj->setIdtipemitmaf($idtipemitmaf);
                    $obj->setVrprececoaf($vrprececoaf);
                    $obj->setVrpreccotaf($vrpreccotaf);
                    $obj->setIdbaixitmaf($idbaixitmaf);
                    $obj->setNrlanctonf($nrlanctonf);
                    $obj->setTxobsitmaf($txobsitmaf);
                    $obj->setVrdescitmaf($vrdescitmaf);
                    $obj->setNrloteestq($nrloteestq);
                    $obj->setNrsublote($nrsublote);
                    $obj->setCdprodprodaf($cdprodprodaf);
                    $obj->setDtentritmsol($dtentritmsol);
                    $obj->setDsinconsitaf($dsinconsitaf);
                    $obj->setIdmotprescap($idmotprescap);
                    $obj->setQtvoluitemaf($qtvoluitemaf);
                    $obj->setIdbaixitmfutura('X');
                    $obj->setIddevolucaonfit('N');

                    $obj->persist();
//                    $connection->flush();
//                    $connection->commit();
//                    \Geral\Util\AcumuladorDeMensagens::getInstance()->addMensagem("TK_RECORD_SAVED", array(), \Geral\Util\Mensagem::VALIDACAO);
                    return true;
                } else {
                    return false;
                }
                // Fim if ($state === 'I' )
            } else {
                // Atualiza no banco, $state = 'U'
                /* @var $obj_ItemAutf Itemautf*/
                $obj_ItemAutf = Itemautf::getObjectByWhere(
                    "NRORG = ? AND CDFILDEST = ? AND CDFILLANC = ? AND NRAUTOFORN = ? AND DTPROGENTAF = ? AND CDPRODUTO = ? AND CDMARCA = ?",
                    array($nrorg, $cdfildest, $cdfillanc, $nrautoforn, $dtprogentaf, $cdproduto, $cdmarca)
                );
                if (is_a($obj_ItemAutf, '\Geral\Entities\Itemautf')) {
//                    $connection = \Setforms2\Shared\Registry::getEntityManager();
//                    $connection->beginTransaction();

                    $obj_ItemAutf->setQtprogentaf($qtprogentaf);
                    $obj_ItemAutf->setVrpreuniaf($vrpreuniaf);
                    $obj_ItemAutf->persist();

//                    $connection->flush();
//                    $connection->commit();
                }
//                \Geral\Util\AcumuladorDeMensagens::getInstance()->addMensagem("TK_RECORD_SAVED", array(), \Geral\Util\Mensagem::VALIDACAO);
                return true;
            }

        } catch (\Exception $e) {
            throw $e;
//            $connection->rollback();
//            \Geral\Util\TratamentoDeMensagens::exibirMensagens($e);
        }

    }


    /**
     * Verifica se o item já foi cadastrado para a AF atual
     * @static
     * @param $nrorg
     * @param $cdfildest
     * @param $cdfillanc
     * @param $nrautoforn
     * @param $dtprogentaf
     * @param $cdproduto
     * @param $cdmarca
     * @return bool
     */
    public static function verificaItemRepetido($nrorg, $cdfildest, $cdfillanc, $nrautoforn, $dtprogentaf,
        $cdproduto, $cdmarca)
    {
        try {
            $obj_ItemAutf = Itemautf::getObjectByWhere(
                "NRORG = ? AND CDFILDEST = ? AND CDFILLANC = ? AND DTPROGENTAF = ? AND NRAUTOFORN = ? AND CDPRODUTO = ? AND CDMARCA = ?",
                array($nrorg, $cdfildest, $cdfillanc, $dtprogentaf, $nrautoforn, $cdproduto, $cdmarca));

            if (is_object($obj_ItemAutf)) {
                /** @var $obj_Produto \Geral\Entities\Produto */
                $obj_Produto = \Geral\Entities\Produto::getObjectByWhere("NRORG = ? AND CDPRODUTO = ?", array($nrorg, $cdproduto));
                /** @var $obj_Marca \Geral\Entities\Marca */
                $obj_Marca = \Geral\Entities\Marca::getObjectByWhere("NRORG = ? AND CDMARCA = ?", array($nrorg, $cdmarca));
                $nome_produto = $obj_Produto->getNmproduto();
                $nome_marca = $obj_Marca->getNmmarca();
                \Geral\Util\AcumuladorDeMensagens::getInstance()->addMensagem(
                    "msg_cmp_produto_marca_item_af_existente", array($nome_produto, $nome_marca), \Geral\Util\Mensagem::ERRO
                );
                return true;

            } else {
                return false;
            }

        } catch (\Exception $e) {
            throw $e;
        }

    }


    /**
     * Consulta nome do produto e nome da marca
     * @param $nrorg
     * @param $cdproduto
     * @param $cdmarca
     * @return array
     */
    private function consultaNomeProdutoMarca($nrorg, $cdproduto, $cdmarca){
        $nome_produto = '';
        $nome_marca   = '';
        /** @var $obj_Produto \Geral\Entities\Produto */
        $obj_Produto = \Geral\Entities\Produto::getObjectByWhere("NRORG = ? AND CDPRODUTO = ?", array($nrorg, $cdproduto));
        /** @var $obj_Marca \Geral\Entities\Marca */
        $obj_Marca = \Geral\Entities\Marca::getObjectByWhere("NRORG = ? AND CDMARCA = ?", array($nrorg, $cdmarca));
        if (is_object($obj_Produto)) {
            $nome_produto = $obj_Produto->getNmproduto();
        }
        if (is_object($obj_Marca)) {
            $nome_marca = $obj_Marca->getNmmarca();
        }
        return array($nome_produto, $nome_marca);

    }


    /**
     * Atualiza a forma de pagamento da AF
     * @static
     * @param $nrorg
     * @param $cdfildest
     * @param $cdfillanc
     * @param $nrautoforn
     * @param $cdformpgto
     * @param $idtpforpgtaf
     * @param $nrdiapgfixaf
     * @param $idutdiapgaf
     * @param $nrparcpgtoaf
     */
    public static function updateFormaPagamento($nrorg, $cdfildest, $cdfillanc, $nrautoforn, $cdformpgto, $idtpforpgtaf,
        $nrdiapgfixaf, $idutdiapgaf, $nrparcpgtoaf){

        if($idtpforpgtaf == 'null'){
            $idtpforpgtaf = null;
        }

        $connection = \Setforms2\Shared\Registry::getEntityManager();
        $connection->beginTransaction();

        /** @var $autoForne Autoforne */
        $autoForne = Autoforne::getObjectByWhere(
            "NRORG = ? AND CDFILDEST = ? AND CDFILLANC = ? AND NRAUTOFORN = ?",
            array($nrorg, $cdfildest, $cdfillanc, $nrautoforn)
        );

        $autoForne->atualizarFormaPagamento($cdformpgto, $idtpforpgtaf, $nrdiapgfixaf, $idutdiapgaf, $nrparcpgtoaf);

        $connection->flush();
        $connection->commit();

        return;
    }


    /**
     * Consulta Autorização de Fornecimento no banco
     * @static
     * @param $nrorg
     * @param $cdfildest
     * @param $cdfillanc
     * @param $nrautoforn
    */
    public static function retornaPrazoPercent($nrorg, $cdfildest, $cdfillanc, $nrautoforn) {
        /* @var $obj_prazopgtaf Prazopgtaf*/
        $obj_prazopgtaf = Prazopgtaf::getObjectsByWhere("NRORG = ? AND CDFILDEST = ? AND CDFILLANC = ? AND NRAUTOFORN = ?",
            array($nrorg, $cdfildest, $cdfillanc, $nrautoforn));
        return $obj_prazopgtaf;
    }


    /**
     * Verifica se o prazo inserido já está presente no banco
     * @static
     * @param $nrorg
     * @param $cdfildest
     * @param $cdfillanc
     * @param $nrautoforn
     */
    public static function verificaPrazo($nrorg, $cdfildest, $cdfillanc, $nrautoforn, $dias)
    {
        /* @var $obj_prazopgtaf Prazopgtaf*/
        $obj_prazopgtaf = Prazopgtaf::getObjectsByWhere("NRORG = ? AND CDFILDEST = ? AND CDFILLANC = ? AND NRAUTOFORN = ? AND INSTR(?, NRDIASPGTOAF) <> 0",
            array($nrorg, $cdfildest, $cdfillanc, $nrautoforn, $dias));
        return $obj_prazopgtaf;
    }


    /**
     * Verifica se há campos obrigatórios vazios na tab Produtos
     * @static
     * @param $data
     * @param $produto
     * @param $marca
     * @return bool
     */
    public static function verificaCampoVazio($data, $produto, $marca)
    {
        if (empty($data)) {
            \Geral\Util\AcumuladorDeMensagens::getInstance()->addMensagem("pla00600_informe_data", array(),  \Geral\Util\Mensagem::ERRO);
            return false;
        } elseif (empty($produto)) {
            \Geral\Util\AcumuladorDeMensagens::getInstance()->addMensagem("pla00600_informe_produto", array(), \Geral\Util\Mensagem::ERRO);
            return false;
        } elseif (empty($marca)) {
            \Geral\Util\AcumuladorDeMensagens::getInstance()->addMensagem("msg_informe_marca", array(), \Geral\Util\Mensagem::ERRO);
            return false;
        }

        return true;
    }


    /**
     * Retorna o código da empresas da filial de destino
     * @static
     * @param $nrorg
     * @param $cdfilial
     * return String
     */
    public static function retornaCdempresa($nrorg, $cdfilial)
    {
        /** @var  $obj_Filial \Geral\Entities\Filial */
        $obj_Filial = \Geral\Entities\Filial::getObjectByWhere("NRORG = ? AND CDFILIAL = ?", array($nrorg, $cdfilial));
        if (!empty($obj_Filial)) {
            return $obj_Filial->getCdempresa();
        }
        return false;
    }

    /**
     * Verifica se o fornecedor está qualificado e se sua qualificação está vencida
     * @static
     * @param $nrorg
     * @param $cdforneced
     * @param $dtautoforn
     */
    public static function validaQualificacao($nrorg, $cdforneced, $dtautoforn)
    {
        /** @var Fornecedor obj_Fornecedor */
        $obj_Fornecedor = Fornecedor::getObjectByWhere("NRORG = ? AND CDFORNECED = ?", array($nrorg, $cdforneced));
        if (is_a($obj_Fornecedor, '\Geral\Entities\Fornecedor')) {
            $cdqualif = $obj_Fornecedor->getCdqualif();
            if (empty($cdqualif)) {
                tk_abort(tk_get_message_by_name("msg_fornecedor_nao_qualificado"));
            }
            $obj_data = $obj_Fornecedor->getDtvalqualif();
            $data = DateUtil::getDataDeString($dtautoforn);
            $anterior = DateUtil::comparaDatas($obj_data, $data, DateUtil::ANTES);
            if ($anterior) {
                tk_abort(tk_get_message_by_name("msg_fornecedor_nao_qualificado"));
            }
        }
    }


    /**
     * Consulta imposto na tabela ITEMAUTF
     * @static
     * @param $nrorg
     * @param $cdfildest
     * @param $cdfillanc
     * @param $nrautoforn
     * @param $cdproduto
     * @param $cdmarca
     * @param $dtprogentaf
     * @return
     */
    public static function consultaItemautf($nrorg, $cdfildest, $cdfillanc, $nrautoforn, $cdproduto, $cdmarca, $dtprogentaf)
    {
        /** @var Itemautf $obj_Itemautf */
        $obj_Itemautf = Itemautf::getObjectByWhere(
            "NRORG = ? AND CDFILDEST = ? AND CDFILLANC = ? AND NRAUTOFORN = ? AND CDPRODUTO = ? AND
             CDMARCA = ? AND DTPROGENTAF = ?",
            array($nrorg, $cdfildest, $cdfillanc, $nrautoforn, $cdproduto, $cdmarca, $dtprogentaf));
        return $obj_Itemautf;
    }


    /**
     * Exclui item da tabela ITEMAUTF
     * @static
     * @param $nrorg
     * @param $cdfildest
     * @param $cdfillanc
     * @param $nrautoforn
     * @param $cdproduto
     * @param $cdmarca
     * @param $dtprogentaf
     * @return bool
     */
    public static function excluiItemautf($nrorg, $cdfildest, $cdfillanc, $nrautoforn, $cdproduto, $cdmarca, $dtprogentaf)
    {
        $connection = \Setforms2\Shared\Registry::getEntityManager();
        $connection->beginTransaction();
        try {
            /** @var Itemautf $obj_Itemautf */
            $obj_Itemautf = self::consultaItemautf($nrorg, $cdfildest, $cdfillanc, $nrautoforn, $cdproduto, $cdmarca, $dtprogentaf);
            if (is_object($obj_Itemautf)) {
                $obj_Itemautf->delete();

                $connection->flush();
                $connection->commit();

                return true;
            }

            return false;
        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;
            return false;
        }
    }


    /**
     * Consulta imposto na tabela IMPITMAF
     * @static
     * @param $nrorg
     * @param $cdfildest
     * @param $cdfillanc
     * @param $nrautoforn
     * @param $cdproduto
     * @param $cdmarca
     * @param $dtprogentaf
     * @param $cdimposto
     * @return Impitmaf
     */
    public static function consultaImpitmaf($nrorg, $cdfildest, $cdfillanc, $nrautoforn, $cdproduto, $cdmarca, $dtprogentaf)
    {
        $objs_Impitmaf = Impitmaf::getObjectsByWhere(
            "NRORG = ? AND CDFILDEST = ? AND CDFILLANC = ? AND NRAUTOFORN = ? AND CDPRODUTO = ? AND CDMARCA = ? AND
            DTPROGENTAF = ?",
            array($nrorg, $cdfildest, $cdfillanc, $nrautoforn, $cdproduto, $cdmarca, $dtprogentaf));
        return $objs_Impitmaf;
    }

    /**
     * Exclui imposto da tabela IMPITMAF
     * @static
     * @param $nrorg
     * @param $cdfildest
     * @param $cdfillanc
     * @param $nrautoforn
     * @param $cdproduto
     * @param $cdmarca
     * @param $dtprogentaf
     * @param $cdimposto
     */
    public static function excluiImpitmaf($nrorg, $cdfildest, $cdfillanc, $nrautoforn, $cdproduto, $cdmarca, $dtprogentaf)
    {
        $connection = \Setforms2\Shared\Registry::getEntityManager();
        try {
            $objs_Impitmaf = self::consultaImpitmaf($nrorg, $cdfildest, $cdfillanc, $nrautoforn, $cdproduto, $cdmarca, $dtprogentaf);
            if (!empty($objs_Impitmaf)) {
                /** @var Impitmaf $obj_Impitmaf */
                foreach ($objs_Impitmaf as $chave => $obj_Impitmaf) {
                    if (is_object($obj_Impitmaf)) {
                        $connection->beginTransaction();
                        $obj_Impitmaf->delete();

                        $connection->flush();
                        $connection->commit();
                    }
                }
            }
        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;
            return false;
        }
    }


    public static function retornaValorTotalProdAf($cdfildest,$cdfillanc,$nrautoforn,$nrorg,$itemautfAlterada,$itemautfNova){
        $soma = 0;
        $encontrou = 0;

        /** @var $itemautf \Geral\Entities\Itemautf*/
        $itemautf = \Geral\Entities\Itemautf::getObjectsByWhere(
            'CDFILDEST = ? AND CDFILLANC = ? AND NRAUTOFORN = ? AND NRORG = ?',
            array($cdfildest,$cdfillanc,$nrautoforn,$nrorg)
        );

        if(empty($itemautf) == false){
            foreach($itemautf as $item){
                $pk = $item->getCdfildest().$item->getCdfillanc().$item->getNrautoforn().$item->getCdmarca().$item->getCdproduto();
                if(empty($itemautfAlterada) == false){
                    foreach($itemautfAlterada as $itemAlterado){
                        $pkNovo = $itemAlterado['CDFILDEST'].$itemAlterado['CDFILLANC'].$itemAlterado['NRAUTOFORN'].$itemAlterado['CDMARCA'].$itemAlterado['CDPRODUTO'];
                        if($pk === $pkNovo){
                            $soma += $itemAlterado['QTPROGENTAF'] * $itemAlterado['VRPREUNIAF'];
                            $encontrou = 1;
                            break;
                        }
                        else{
                            $encontrou = 0;
                        }
                    }
                    if($encontrou == 0){
                        $soma += $item->getQtprogentaf() * $item->getVrpreuniaf();
                    }
                } else{
                    $soma += $item->getQtprogentaf() * $item->getVrpreuniaf();
                }
            }
        }
        if (!empty($itemautfNova)) {
            foreach($itemautfNova as $itemNova){
                $soma += $itemNova['QTPROGENTAF'] * $itemNova['VRPREUNIAF'];
            }
        }
        return $soma;
    }


    public static function retornaValorTotalReal($cdfildest,$cdfillanc,$nrautoforn,$nrorg){
        /** @var  $autoforne  Autoforne*/
        $autoforne = Autoforne::getObjectByWhere(
            'CDFILDEST = ? AND CDFILLANC = ? AND NRAUTOFORN = ? AND NRORG = ?',
            array($cdfildest,$cdfillanc,$nrautoforn,$nrorg)
        );

        $valorTotalReal = $autoforne->getVrtotalaforg();
        if(empty($valorTotalReal)){
            $valorTotalReal = $autoforne->getVrtotalaf();
            $autoforne->setVrtotalaforg($valorTotalReal);
            $autoforne->persist();

            \Setforms2\Shared\Registry::getEntityManager()->flush();
        }

        return $valorTotalReal;

    }


    public static function retornaValorAcrescimoAf($cdfilial,$valorTotalReal,$valorTotalNovo,$nrorg){
        /** @var  $parcmpsup Parcmpsup*/
        $parcmpsup = Parcmpsup::getObjectByWhere(
            'CDFILIAL = ? AND NRORG = ?',
            array($cdfilial,$nrorg)
        );

        $acrescimoAf = $parcmpsup->getVracrescimoaf();

        $valorTotalReal += $valorTotalReal*$acrescimoAf/100;
        if($valorTotalNovo > $valorTotalReal){
            return $valorTotalReal;
        } else{
            return true;
        }

    }

}